//****************************************************************************
//**
//**    M_MISC.CPP
//**    Miscellaneous Utility Functions
//**
//****************************************************************************
//============================================================================
//    IMPLEMENTATION HEADERS
//============================================================================
#define SYS_DEFS_INCLUDE_WIN32 // for SYSW_TimeFloat
#include "sys_defs.h"
#include "m_misc.h"

//============================================================================
//    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
//============================================================================
//============================================================================
//    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
//============================================================================
//============================================================================
//    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
//============================================================================
//============================================================================
//    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
//============================================================================
//============================================================================
//    IMPLEMENTATION PRIVATE DATA
//============================================================================
//============================================================================
//    INTERFACE DATA
//============================================================================
//============================================================================
//    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
//============================================================================
//============================================================================
//    IMPLEMENTATION PRIVATE FUNCTIONS
//============================================================================
//============================================================================
//    INTERFACE FUNCTIONS
//============================================================================

// Return program time in seconds
float M_TimeFloat()
{
	// cascade to windows function for this, as it uses windows performance
	// counter and multimedia timer functions
	return(SYSW_TimeFloat());
}

// Check for command line option (case insensitive), prefixed with - or /
// Returns SYS_Argv() index of option if found, 0 if not
// If reqParms != 0, makes sure additional parameters follow the index
byte M_CmdLineOption(char* str, byte reqParms)
{
	byte i;
	for (i=1;i<(SYS_Argc()-reqParms);i++)
	{
		if ( ((SYS_Argv(i)[0] == '-') || (SYS_Argv(i)[0] == '/'))
		  && (!stricmp(&SYS_Argv(i)[1], str)) )
		{
			return(i);
		}
	}
	return(0);
}

// Get filename path directory
// Result is in internal buffer; copy if need to preserve
char* M_GetFilePath(char* filename)
{
	static char pathBuf[SYS_MAX_PATH];
	char *ptr;

	strcpy(pathBuf, filename);
	ptr = strrchr(pathBuf, '\\');
	if (ptr)
	{
		ptr[1] = 0;
		return(pathBuf);
	}
	ptr = strrchr(pathBuf, ':');
	if (ptr)
	{
		ptr[1] = '\\';
		ptr[2] = 0;
		return(pathBuf);
	}
	pathBuf[0] = 0;
	return(pathBuf);
}

// Get filename without path or extension
// Result is in internal buffer; copy if need to preserve
char* M_GetFileRoot(char* filename)
{
	static char rootBuf[SYS_MAX_PATH];
	char *ptr;

	ptr = strrchr(filename, '\\');
	if (!ptr)
		ptr = strrchr(filename, ':');
	if (!ptr)
		ptr = filename-1;
	strcpy(rootBuf, ptr+1);
	ptr = strchr(rootBuf, '.');
	if (ptr)
		*ptr = 0;
	return(rootBuf);
}

// Get filename extension (without the '.')
char* M_GetFileExtension(char* filename)
{
	char *ptr = strrchr(filename, '.');
	if (ptr)
		ptr++;
	else
		ptr = "";
	return(ptr);
}

// Add extension if one isn't present.  Extension parameter should be w/o '.'
void M_SuggestFileExtension(char* filename, char* extension)
{
	char *ptr = strrchr(filename, '.');
	if (!ptr)
	{
		strcat(filename, ".");
		strcat(filename, extension);
	}
}

// Set the extension even if one is there.  Extension should be w/o '.'
void M_ForceFileExtension(char* filename, char* extension)
{
	char *ptr = strrchr(filename, '.');
	if (ptr)
		*ptr = 0;
	strcat(filename, ".");
	strcat(filename, extension);
}

//============================================================================
//    INTERFACE CLASS BODIES
//============================================================================

//****************************************************************************
//**
//**    END MODULE M_MISC.CPP
//**
//****************************************************************************

