#ifndef __SYS_MAIN_H__
#define __SYS_MAIN_H__
//****************************************************************************
//**
//**    SYS_MAIN.H
//**    Header - System Main
//**
//****************************************************************************
//============================================================================
//    INTERFACE REQUIRED HEADERS
//============================================================================
#include "sys_defs.h"

//============================================================================
//    INTERFACE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
//============================================================================
// convenience macros
#define SYS_MALLOC(type, num) (type *)SYS_SafeMalloc((num)*sizeof(type))
#define SYS_FREE(ptr) SYS_SafeFree((void **)&ptr)

//============================================================================
//    INTERFACE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
//============================================================================
//============================================================================
//    INTERFACE STRUCTURES / UTILITY CLASSES
//============================================================================
//============================================================================
//    INTERFACE DATA DECLARATIONS
//============================================================================
extern char sys_programPath[SYS_MAX_PATH]; // current directory at runtime
extern float sys_curTime; // time at start of current frame
extern float sys_frameTime; // time in seconds between current and last frame

//============================================================================
//    INTERFACE FUNCTION PROTOTYPES
//============================================================================
// Primary control functions
void SYS_Init(); // initialization
void SYS_Frame(); // main loop
void SYS_Shutdown(); // deinitialization

// System output
void SYS_Alertf(char* text, ...);

// Termination etc.
void SYS_Quit(); // terminate normally
void SYS_Errorf(char* text, ...); // terminate with an error

// Command line arguments
byte SYS_Argc(); // argument count
char* SYS_Argv(byte index); // argument strings

// Library memory management
void* SYS_SafeMalloc(int size); // malloc() wrapper
void SYS_SafeFree(void **ptr); // free() wrapper

//============================================================================
//    INTERFACE OBJECT CLASS DEFINITIONS
//============================================================================
//============================================================================
//    INTERFACE TRAILING HEADERS
//============================================================================

//****************************************************************************
//**
//**    END HEADER SYS_MAIN.H
//**
//****************************************************************************
#endif // __SYS_MAIN_H__
