//****************************************************************************
//**
//**    SYS_WIN.CPP
//**    Windows System
//**
//****************************************************************************
//============================================================================
//    IMPLEMENTATION HEADERS
//============================================================================
#define SYS_DEFS_INCLUDE_WIN32
#include "sys_defs.h"
#include "sys_win.h"

//============================================================================
//    IMPLEMENTATION PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
//============================================================================
#define MAINWINDOW_TITLE APPTITLE
#define MAINWINDOW_CLASS APPTITLE "_MAINWINDOW"

//============================================================================
//    IMPLEMENTATION PRIVATE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
//============================================================================
//============================================================================
//    IMPLEMENTATION PRIVATE STRUCTURES / UTILITY CLASSES
//============================================================================
//============================================================================
//    IMPLEMENTATION REQUIRED EXTERNAL REFERENCES (AVOID)
//============================================================================
//============================================================================
//    IMPLEMENTATION PRIVATE DATA
//============================================================================
// command line
byte sys_argc;
char** sys_argv;

// timer
static sqword sys_ticks_per_sec;
static float sys_secs_per_tick;
static dwbool sys_PerformanceTimerEnabled;
static sqword sys_PerformanceTimerStart;
static dword sys_MMTimerStart;

// exception handling
static dwbool sys_exceptionhandler = 1;

//============================================================================
//    INTERFACE DATA
//============================================================================
HINSTANCE sysw_hInst;
HWND sysw_hwnd;

//============================================================================
//    IMPLEMENTATION PRIVATE FUNCTION PROTOTYPES
//============================================================================
static void SetupTimeFloat();
static SYSW_WINDOWFUNC(MainWindow_wf);
static int EvaluateException(unsigned long exceptnum);
static void ExceptionHandler(unsigned long e);
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst,
				   LPSTR argList, int winMode);

//============================================================================
//    IMPLEMENTATION PRIVATE FUNCTIONS
//============================================================================
static void SetupTimeFloat()
{
	if (!QueryPerformanceFrequency((LARGE_INTEGER *)&sys_ticks_per_sec))
	{ // no performance counter available
		sys_MMTimerStart = timeGetTime();
		sys_secs_per_tick = 1.0f/1000.0f;
		sys_PerformanceTimerEnabled = 0;
	}
	else
	{ // performance counter is available, use it instead of multimedia timer
		QueryPerformanceCounter((LARGE_INTEGER *)&sys_PerformanceTimerStart);
		sys_secs_per_tick = ((double)1.0)/((double)sys_ticks_per_sec);
		sys_PerformanceTimerEnabled = 1;
	}
}

static int RegisterMainWindowClass()
{
	WNDCLASSEX wcl;

	wcl.style = CS_DBLCLKS;
	wcl.cbSize = sizeof(WNDCLASSEX);
	wcl.hIcon = LoadIcon(sysw_hInst, MAKEINTRESOURCE(IDI_APPLICATION));
	wcl.hIconSm = LoadIcon(sysw_hInst, MAKEINTRESOURCE(IDI_APPLICATION));
	wcl.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcl.lpszMenuName = NULL;
	wcl.cbClsExtra = 0;
	wcl.cbWndExtra = 0;
	wcl.hInstance = sysw_hInst;
	wcl.lpszClassName = MAINWINDOW_CLASS;
	wcl.lpfnWndProc = MainWindow_wf;
	wcl.hbrBackground = (HBRUSH)GetStockObject(DKGRAY_BRUSH);
	if (!RegisterClassEx(&wcl))
		return(0);
	return(1);
}

static SYSW_WINDOWFUNC(MainWindow_wf)
{
	switch(msg)
	{
	case WM_CLOSE:
	case WM_DESTROY:
	case WM_QUIT:
        SYS_Quit();
		break; // not like this will ever be reached, but i like clean switches
	case WM_CHAR:
		if (wParam == VK_ESCAPE)
			SYS_Quit(); // placeholder until user input system is in place
		break;
	default:
		break;
	}
	return(DefWindowProc(hwnd, msg, wParam, lParam));
}

static int EvaluateException(unsigned long exceptnum)
{	
	if ( exceptnum == STATUS_INTEGER_OVERFLOW ||
		 exceptnum == STATUS_FLOAT_UNDERFLOW ||
		 exceptnum == STATUS_FLOAT_OVERFLOW )
		 return(EXCEPTION_CONTINUE_SEARCH);

	if (sys_exceptionhandler)
	{ // exception handler is active, try to shut everything down and let the error get caught
		return(EXCEPTION_EXECUTE_HANDLER);
	}
	else
	{ // no exception handler, let it slide through (if the debugger is active, it should catch it)
		return(EXCEPTION_CONTINUE_SEARCH);
	}
}

static void ExceptionHandler(unsigned long e)
{
	static char *disclaimer = "\nThis program has hit a critical error.  Please notify the author with the above information.";
	switch(e)
	{
	case EXCEPTION_ACCESS_VIOLATION:			SYS_Errorf("Fatal Exception: Access Violation\n%s\n", disclaimer); break;
	case EXCEPTION_ARRAY_BOUNDS_EXCEEDED:		SYS_Errorf("Fatal Exception: Array Bounds Exceeded\n%s\n", disclaimer); break;
	case EXCEPTION_BREAKPOINT:					SYS_Errorf("Fatal Exception: Breakpoint\n%s\n", disclaimer); break;
	case EXCEPTION_DATATYPE_MISALIGNMENT:		SYS_Errorf("Fatal Exception: Datatype Misalignment\n%s\n", disclaimer); break;
	case EXCEPTION_FLT_DENORMAL_OPERAND:		SYS_Errorf("Fatal Exception: Float Denormal Operand\n%s\n", disclaimer); break;
	case EXCEPTION_FLT_DIVIDE_BY_ZERO:			SYS_Errorf("Fatal Exception: Float Divide By Zero\n%s\n", disclaimer); break;
	case EXCEPTION_FLT_INEXACT_RESULT:			SYS_Errorf("Fatal Exception: Float Inexact Result\n%s\n", disclaimer); break;
	case EXCEPTION_FLT_INVALID_OPERATION:		SYS_Errorf("Fatal Exception: Float Invalid Operation\n%s\n", disclaimer); break;
	case EXCEPTION_FLT_OVERFLOW:				SYS_Errorf("Fatal Exception: Float Overflow\n%s\n", disclaimer); break;
	case EXCEPTION_FLT_STACK_CHECK:				SYS_Errorf("Fatal Exception: Float Stack Check\n%s\n", disclaimer); break;
	case EXCEPTION_FLT_UNDERFLOW:				SYS_Errorf("Fatal Exception: Float Underflow\n%s\n", disclaimer); break;
	case EXCEPTION_ILLEGAL_INSTRUCTION:			SYS_Errorf("Fatal Exception: Illegal Instruction\n%s\n", disclaimer); break;
	case EXCEPTION_IN_PAGE_ERROR:				SYS_Errorf("Fatal Exception: In-page Error\n%s\n", disclaimer); break;
	case EXCEPTION_INT_DIVIDE_BY_ZERO:			SYS_Errorf("Fatal Exception: Integer Divide By Zero\n%s\n", disclaimer); break;
	case EXCEPTION_INT_OVERFLOW:				SYS_Errorf("Fatal Exception: Integer Overflow\n%s\n", disclaimer); break;
	case EXCEPTION_INVALID_DISPOSITION:			SYS_Errorf("Fatal Exception: Invalid Disposition\n%s\n", disclaimer); break;
	case EXCEPTION_NONCONTINUABLE_EXCEPTION:	SYS_Errorf("Fatal Exception: Noncontinuable Exception\n%s\n", disclaimer); break;
	case EXCEPTION_PRIV_INSTRUCTION:			SYS_Errorf("Fatal Exception: Private Instruction\n%s\n", disclaimer); break;
	case EXCEPTION_SINGLE_STEP:					SYS_Errorf("Fatal Exception: Single-step\n%s\n", disclaimer); break;
	case EXCEPTION_STACK_OVERFLOW:				SYS_Errorf("Fatal Exception: Stack Overflow\n%s\n", disclaimer); break;
	default:									SYS_Errorf("Fatal Exception: Unknown: %x\n%s\n", e, disclaimer); break;
	}
}

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst,
				   LPSTR argList, int winMode)
{
	MSG msg;
	unsigned long exceptval;

	sysw_hInst = hInst; // set up global instance
	sys_argc = __argc; // command line arguments
	sys_argv = __argv;
	_getcwd(sys_programPath, SYS_MAX_PATH); // execution directory
	
	SetupTimeFloat(); // initialize system timer

	// register the main window class
	if (!RegisterMainWindowClass())
		SYS_Errorf("WinMain: Unable to register main window class");

	// Create main window
	sysw_hwnd = CreateWindowEx(WS_EX_APPWINDOW, MAINWINDOW_CLASS, MAINWINDOW_TITLE,
		WS_POPUP | WS_OVERLAPPED | WS_CAPTION | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_MAXIMIZE,
		//0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN),
		CW_USEDEFAULT, CW_USEDEFAULT, 640, 480,
		HWND_DESKTOP, NULL, sysw_hInst, NULL);
	if (!sysw_hwnd)
		return(false);

	// Display main window
	ShowWindow(sysw_hwnd, winMode);
	UpdateWindow(sysw_hwnd);

	// Call application startup stuff
	SYS_Init();

	// Main loop
	__try
	{
		while (1)
		{
			if(PeekMessage(&msg,sysw_hwnd, 0, 0, PM_REMOVE))
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
			SYS_Frame(); // call application main loop function
		}
	}
	__except(EvaluateException(exceptval = GetExceptionCode()))
	{ // A Bad Thing happened, try to crash cleanly
		ExceptionHandler(exceptval);
	}

	// Shouldn't ever get here, but put this here for safety
	SYS_Shutdown();

	return(msg.wParam);
}

//============================================================================
//    INTERFACE FUNCTIONS
//============================================================================

// get command line argument count
byte SYSW_Argc()
{
	return(sys_argc);
}

// get command line argument string
char* SYSW_Argv(byte index)
{
	if (index >= sys_argc)
		SYS_Errorf("SYSW_Argv: Invalid index %d, maximum is %d", index, sys_argc-1);
	return(sys_argv[index]);
}

// time passed in program in seconds
float SYSW_TimeFloat()
{
	sqword time;

	if (sys_PerformanceTimerEnabled)
	{
		QueryPerformanceCounter((LARGE_INTEGER *)&time);
		return(((float)(time-sys_PerformanceTimerStart)) * sys_secs_per_tick);
	}
	else
	{
		return(((float)(timeGetTime()-sys_MMTimerStart)) * sys_secs_per_tick);
	}
}

// windows messagebox displaying
int SYSW_MessageBox(char* caption, unsigned long boxType, char* text, ... )
{
	static char textBuffer[512];
	va_list args;
	va_start(args, text);
	vsprintf(textBuffer, text, args);
	va_end(args);
	return(MessageBox(NULL, textBuffer, caption, boxType));
}

//============================================================================
//    INTERFACE CLASS BODIES
//============================================================================

//****************************************************************************
//**
//**    END MODULE SYS_WIN.CPP
//**
//****************************************************************************

